///////////////////////////////////////////////////////////////////////
// Keypad.cpp
// 720020-1161x
// Keypad Helper Class
//
// Applied Data Systems
// www.applieddata.net
//
// Description
// -----------
// This helper class helps provide keyboard functionality from a digital 
// keypad using the ADSmartIO library.  A key map is passed into the class
// and this map converts keypad button presses to virtual key presses. For 
// virtual key value definitions, see <winuser.h>.
//
// Notes
// -----
// The ADSmartIO keypad functions support only one key press at a time.  This
// precludes using key combinations, like shift-a or ctrl-c.  As a "workaround"
// to this problem, this library provides functionality similar to the soft
// input panel (SIP).  If you press a SHIFT key, then the next key will have
// SHIFT applied.  Once the shifted key is sent, the "shift" will be turned
// back off.  This functionality exists for the SHIFT and CONTROL keys only.
//
// This library also depends on the ADS ADSmartIO helper class (720020-1153x).
//
// History
// -------
// 720020-11611 - Oct 14, 2003 : ctacke
//   - Initial release
//
//
///////////////////////////////////////////////////////////////////////// 
#include "keypad.h"
#include "adsmartio.h"

#define STOP_EVENT _T("KEYPAD_STOP")

Keypad::MAPKEY  *Keypad::m_keymap = NULL;
int				Keypad::m_cols = 0;
int				Keypad::m_rows = 0;
int				Keypad::m_keycount = 0;


Keypad::Keypad()
{
	// this event will be used to stop the worker thread
	m_hStopEvent = CreateEvent(NULL, TRUE, FALSE, STOP_EVENT);
	m_threadID = 0;
}

Keypad::~Keypad()
{
	// stop any worker thread
	SetEvent(m_hStopEvent);
}

void Keypad::Start()
{
	// only do something if a worker isn't running
	if(! m_threadID)
	{
		// start a worker thread
		CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)KeyThreadProc, NULL, 0, &m_threadID);
	}
}

void Keypad::Stop()
{
	// stop any worker thread
	SetEvent(m_hStopEvent);

	m_threadID = 0;
}

DWORD WINAPI Keypad::KeyThreadProc(LPVOID params)
{
	BYTE key = 0;
	bool shift = false;
	bool control = false;

	// create a smartio object for keypad support
	ADSmartIO *smartio = new ADSmartIO();

	// create a listener event for the Stop method
	HANDLE hStopEvent = CreateEvent(NULL, TRUE, FALSE, STOP_EVENT);

	// select the option for a keypad
	smartio->SelectOption(3);

	// set the keypad size
	smartio->SetKeyPadSize(m_rows, m_cols);

	// start our key listener
	while(true)
	{
		// check to see if stop has been called (or our object died)
		if(WaitForSingleObject(hStopEvent, 0) == WAIT_OBJECT_0)
			break;

		// read a key stroke
		key = smartio->ReadKeypad();
		
		// ignore timeouts
		if(key == 255)
			continue;

		// check the map for the key
		for(int i = 0 ; i < m_keycount ; i++)
		{
			// look for a mapped key
			if(key == m_keymap[i].KeypadVal)
			{
				// see if it's mapped for special functionality (shift, etc.)
				if(m_keymap[i].VirtualKey == VK_SHIFT)
				{
					shift = !shift;
					break;
				}

				if(m_keymap[i].VirtualKey == VK_CONTROL)
				{
					control = !control;
					break;
				}


				if(shift)
				{
					// key down
					keybd_event(VK_SHIFT, 1, 0, 0);
				}
				if(control)
				{
					// key down
					keybd_event(VK_CONTROL, 1, 0, 0);
				}

				// key down
				keybd_event(m_keymap[i].VirtualKey, 1, 0, 0);
				// key up
				keybd_event(m_keymap[i].VirtualKey, 1, KEYEVENTF_KEYUP, 0);

				// reset the special keys
				if(shift)
				{
					// key up
					keybd_event(VK_SHIFT, 1, KEYEVENTF_KEYUP, 0);
					shift = false;
				}
				if(control)
				{
					// key up
					keybd_event(VK_CONTROL, 1, KEYEVENTF_KEYUP, 0);
					control = false;
				}

				break;
			}
		}
	}

	// clean up
	delete smartio;

	return 0;
}

void Keypad::SetMap(MAPKEY keymap[],  int rows, int columns)
{
	// save the row count
	m_rows = rows;
	// and the column count
	m_cols = columns;
	//calc the key count
	m_keycount = m_rows * m_cols;
	//save a pointer to the map
	m_keymap = keymap;

	// under debug, show the keymap
	DEBUGMSG(TRUE, (_T("%i keys mapped as follows:\r\n"), m_keycount));
	for(int i = 0 ; i < m_keycount ; i++)
	{
		DEBUGMSG(TRUE, (_T("Keypad 0x%02x = VK 0x%02x\r\n"), 
				m_keymap[i].KeypadVal,  
				m_keymap[i].VirtualKey ));
	}
}